/*
 * geography.h - general structures and constants to use in geography
 *
 * Copyright (C) 1999 Gordon D. Carrie
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 * 
 * Please address questions and bug reports to gcarri@mail.txcyber.com
 *
 ********************************************
 *
 */

#ifndef _GEOGRAPHY_H_
#define _GEOGRAPHY_H_

#include <float.h>

/* Constants to use in round Earth calculations */
#define RADPERDEG 0.017453292519943294892
#define DEGPERRAD 57.29577951308232088
#define DEG_INV 0.002777777777777778

/* Earth dimensions, in centimeters */
/* Radius is such that 1.0 nautical mile = 1' latitude = 185200 cm */
#define REARTH 636670701.949
#define REARTH_INV 1.57067067314e-09
#define CMPERDEG 11112000.0
#define DEGPERCM 8.9992800576e-08

/* Some conversion factors */
#define NMIPERDEG 60.0
#define SMIPERDEG 69.0468
#define KMPERDEG 111.12

/* Convert number of degrees to integer number of "degree units", where 
 * there are UNITSPERDEG units per degree, resulting in an absolute precision
 * of DEGPREC */
#define UNITSPERDEG 1000
#define DEGPREC 0.0005
#define DEGUNITS(deg) \
        ((int)(UNITSPERDEG * ((deg) < 0 ? (deg) - DEGPREC : (deg) + DEGPREC)))

/* Degree units in a great circle and a hemisphere */
#define UNITSPERGC 360000
#define UNITSPERGC_2 180000

/* A geographical point */
typedef struct
{
  float			ord;		/* Ordinate */
  float			abs;		/* Abscissa */
} GeoPoint;

/* Point in Geocentric Cartesian Coordinates, unit radius */
typedef struct
{
  double		x;
  double		y;
  double		z;
} CartPoint;

/* A geographical error */
#define NOWHERE {FLT_MAX, FLT_MAX}

/* Opposite directions.  Use for comparisons and to identify
 * hemispheres.  */
enum LatSgn {North, Eq, South};
enum LonSgn {West, PrMd, East};

/* Latitude-longitude limits */
typedef struct
{
  double north;
  double east;
  double south;
  double west;
} Bounds;

/* Convert a latitude-longitude to Geocentric Cartesian Coordinates */
/* Earth has unit radius */
CartPoint LatLonToCart(GeoPoint geopt);

/* Put a longitude into range [refLon - 180, refLon + 180] */
/* Can sometimes make arithmetic and comparisons easier with longitudes */
double DomainLon(double lon, double refLon);
double GwchLon(double lon);
GeoPoint DomainLonPt(GeoPoint geoPoint, double refLon);
GeoPoint GwchLonPt(GeoPoint geoPoint);

/* Determine if lon0 is west, east, or the same as lon1 */
enum LonSgn LonCmp(double lon0, double lon1);

/* Determine if lat0 is south, north, or the same as lat1 */
enum LatSgn LatCmp(double lat0, double lat1);

/* Determine if two degree measures are the same */
int DegCmp(double d0, double d1);

/* Determine if a longitude is between two other longitudes */
int LonBtwn(double lon, double lon0, double lon1);
int LonBtwn1(double lon, double lon0, double lon1);

/* Step distance in direction from a point */
GeoPoint GeoStep(GeoPoint pnt, double dir, double dist);

/* Distance between latlons in Great Circle degrees */
double GeoDistance(GeoPoint p1, GeoPoint p2);

/* Distance between latlons as square of absolute separation */
double GeoQuickDistance(GeoPoint p1, GeoPoint p2);

/* Find intersection of two lines, given two points on each line */
GeoPoint LinearX(GeoPoint geoPt1, GeoPoint geoPt2,
  GeoPoint geoPt3, GeoPoint geoPt4);

/* Find where great circle crosses latitude */
int GCircleXLatCircle(GeoPoint geoPt0, GeoPoint geoPt1, double lat,
                      double *lon0, double *lon1);

/* Find intersections of two great circles, given two points on each circle */
GeoPoint GCircleX(GeoPoint ln1pt1, GeoPoint ln1pt2, GeoPoint ln2pt1,
  GeoPoint ln2pt2);

#endif
